#
# Copyright (C) 2012 INRA
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import inspect

from ConfigParser import ConfigParser, NoOptionError

from jflow.utils import which


class JFlowConfigReader(object):
    """
    """
    
    CONFIG_FILE_PATH = "../../application.properties"
    
    def __init__(self):
        """ 
        """
        self.reader = ConfigParser()
        self.reader.read(os.path.join(os.path.dirname(inspect.getfile(self.__class__)), self.CONFIG_FILE_PATH))

    def get_tmp_directory(self):
        return self.reader.get("storage", "tmp_directory")
        
    def get_work_directory(self):
        return self.reader.get("storage", "work_directory")
    
    def get_exec(self, software):
        try:
            return self.reader.get("softwares", software)
        except NoOptionError, e:
            return None

    def get_resource(self, resource):
        return self.reader.get("resources", resource)

    def get_log_file_path(self):
        """
        return the log file path
          @return: the path to the log file
        """
        try:
            return self.reader.get('storage', 'log_file')
        except :
            raise Error("Failed when parsing the config file, no section logging found!")
        
    def get_makeflow_path(self):
        try:
            exec_path = self.reader.get("global", "makeflow")
        except NoOptionError, e:
            exec_path = None
        if exec_path is None: exec_path = "makeflow"
        if which(exec_path) == None:
            sys.stderr.write("Error: 'makeflow' path connot be retrieved either in the PATH and in the application.properties file!\n")
            sys.exit(1)
        return exec_path
    
    def get_batch(self):
        try:
            type = self.reader.get("global", "batch_system_type")
            options = self.reader.get("global", "batch_options")
            return [type, options]
        except NoOptionError, e:
            return None
          
    def get_socket_options(self):
        try:
            return [self.reader.get("global", "server_socket_host"), int(self.reader.get("global", "server_socket_port"))]
        except:
            return ["127.0.0.1", 8080]
          
    def get_component_batch_options(self, component_class):
        try:
            return self.reader.get("components", component_class+".batch_options")
        except:
            return ""
        