#
# Copyright (C) 2012 INRA
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

def which(program):
    """
    Return if the asked program exist in the user path
      @param options : the options asked by the user
    """
    import os
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK)
    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None

def get_file_base (file):
    """
    Return the file base
      @param file: input file
    """
    # First import required libraries
    import os
    
    file_base = os.path.splitext(os.path.basename(file))[0]
    if os.path.splitext(file)[1] == ".gz":
        file_base = os.path.splitext(file_base)[0]
    return file_base

def get_nb_string(value, length=6):  
    """
    Return a string of n caracters.
      @param value: the value to get representation
      @param length: the final length required
    """
    zeros = ""
    s_value = str(value)
    for i in range(length - len(s_value)):
        zeros += "0";
    s_value = zeros + s_value
    return s_value

def get_argument_pattern( list, start_number=1 ):
    """
    Return the argument pattern for arguments files. Ex : with 3 element in list this function returns ["${1} ${2} ${3}", 4].
      @param list : the list of files.
      @param start_number : number of the first argument.
    """
    arg_pattern = ""
    next_number = start_number
    for elt in list:
        arg_pattern += ' ${' + str(next_number) + '}'
        next_number += 1
        
    return [arg_pattern, next_number]