/***************************************************************
*  Copyright notice
*
*  (c) 2013 PF bioinformatique de Toulouse
*  All rights reserved
* 
*  It is distributed under the terms of the GNU General Public 
*  License as published by the Free Software Foundation; either 
*  version 2 of the License, or (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

﻿$.validator.setDefaults({
    highlight: function (element, errorClass, validClass) {
    	var $element;
		if ( element.type === "radio" ) {
			$element = this.findByName(element.name);
		} else {
			$element = $(element);
		}
        $element.addClass(errorClass).removeClass(validClass);
        // add the bootstrap error class
		if ($element.parent().parent("blockquote").find("div").length > 0) {
			// if it's a multiple type
			$element.parent().parent("blockquote").parent("div.controls").parent("div.control-group").addClass(errorClass);
			//highlight append button as well$
			$element.parent().parent("blockquote").find("div.input-append").each(function() {
				$(this).find(".btn").addClass("btn-danger");
				$(this).find(".icon-calendar").addClass("icon-white");
			});
		} else {
			$element.parents("div.control-group").addClass(errorClass);
		}
        // if it's a calendar
		if ($element.parent("div.input-append").find(".btn").length > 0) {
			$element.parent("div.input-append").find(".btn").addClass("btn-danger");
			$element.parent("div.input-append").find(".icon-calendar").addClass("icon-white");
		}
		
    },
    unhighlight: function (element, errorClass, validClass) {
        var $element;
        if (element.type === 'radio') {
            $element = this.findByName(element.name);
        } else {
            $element = $(element);
        }
        $element.removeClass(errorClass).addClass(validClass);
        // remove the bootstrap error class
        if ($element.parent().parent("blockquote").find("div").length > 0) {
        	// if it's a multiple type, check if there is no other error
			if ($element.parent().parent("blockquote").find("input." + errorClass).length == 0) {
				$element.parents("div.control-group").removeClass(errorClass).addClass(validClass);
				$element.parent().parent("blockquote").find("div.input-append").each(function() {
					$(this).find(".btn").removeClass("btn-danger");
					$(this).find(".icon-calendar").removeClass("icon-white");
				});
			}
		} else if ($element.parents("div.control-group").find("input." + errorClass).length == 0) {  
            // Only remove the class if there are no other errors
            $element.parents("div.control-group").removeClass(errorClass).addClass(validClass);
            $element.parents(errorClass).removeClass(errorClass);
            if ($element.parent("div.input-append").find(".btn").length > 0) {
    			$element.parent("div.input-append").find(".btn").removeClass("btn-danger");
    			$element.parent("div.input-append").find(".icon-calendar").removeClass("icon-white");
    		}
        }	
    },
    errorPlacement: function(error, element) {
    	error.addClass("help-block");
    	if (element.parent("div.input-append").length > 0) {
    		error.insertAfter(element.parent("div.input-append"));
    	} else if (element.parent("div.input-prepend").length > 0) {
    		error.insertAfter(element.parent("div.input-prepend"));
    	} else {
    		error.insertAfter(element);
    	}
    },
    errorElement: "span",
    errorClass: "error"
});

jQuery.validator.addMethod("exlude", function(value, element, options) {
	var validator = this,
		selector = options;
	var numberFilled = $(selector, element.form).filter(function() {
		return validator.elementValue(this);
	}).length;
	var valid = numberFilled === 1;

	if(!$(element).data('being_validated')) {
		var fields = $(selector, element.form);
		fields.data('being_validated', true);
		fields.valid();
		fields.data('being_validated', false);
	}
	return valid;
}, jQuery.format("Please fill only one of these fields."));

!function ($) {

	"use strict"; // jshint ;_;


	/* WFForm CLASS DEFINITION
	 * ========================= */

	var WFForm = function (element, options) {
		this.$element = $(element)
		this.options = $.extend({}, $.fn.wfform.defaults, options);
		if (this.options.serverURL == "") { this.options.serverURL = $.fn.activewf.defaults.serverURL; }
	}

	WFForm.prototype.reset = function() {
		for (var i in this.workflow.parameters) {
			if (this.workflow.parameters[i].default) {
    			$("#"+this.workflow.parameters[i].name).val(this.workflow.parameters[i].default);
    		} else {
    			$("#"+this.workflow.parameters[i].name).val("");
    		}
		}
	}
	
	WFForm.prototype.run = function() {
		// first check if the form is valid
		if ($("#workflow_form").valid()) {
			// change boolean type values if checked
			$(":checkbox").each(function(){
				if ($(this).prop('checked')) { $(this).val(true); }
			})
			// then send data
			var params = "",
				$this = this;
			$.each ( $('#workflow_form').serializeArray(), function(_, kv) {
				params += kv.name + "=" +escape(kv.value) + "&";
			});
			$("#workflow_form :checkbox").each(function(){
				if (!$(this).prop('checked')) { params += $(this).attr("name") + "=false&"; }
			})
			$.ajax({
			    url: this.options.serverURL + '/run_workflow?' + params + 'callback=?',
			    dataType: "json",
			    success: function(running_wf) {
			    	$this.$element.trigger('run', running_wf);
			    }
	    	});
		}
	}
	
	WFForm.prototype.load = function() {
		var $this = this,
			waiting_animation = ['<div class="row-fluid"><div class="span1 offset3"><div class="inline floatingBarsG">',
		                         '<div class="blockG" id="rotateG_01"></div>',
		                         '<div class="blockG" id="rotateG_02"></div>',
		                         '<div class="blockG" id="rotateG_03"></div>',
		                         '<div class="blockG" id="rotateG_04"></div>',
		                         '<div class="blockG" id="rotateG_05"></div>',
		                         '<div class="blockG" id="rotateG_06"></div>',
		                         '<div class="blockG" id="rotateG_07"></div>',
		                         '<div class="blockG" id="rotateG_08"></div>',
		                         '</div></div> <div class="span6">Please wait until modul is being loaded!</div></div>'].join('\n');
		$this.$element.html(waiting_animation);
		$.ajax({
		    url: this.options.serverURL + '/get_available_workflows?callback=?',
		    dataType: "json",
		    timeout: 20000,
		    error: function (xhr, ajaxOptions, thrownError) {
		    	var alert_message = ['<div class="alert alert-error">',
		        					'<strong>Error!</strong>',
		        					'Jflow failed to connect to the specified server <strong>' + $this.options.serverURL + '</strong>',
		        					'</div>'].join('\n');
		        $this.$element.html(alert_message);
		    },
		    success: function(data) {
		    	var rules = {};
		    	for (var i in data) {
		    		if (data[i]["class"] == $this.options.workflowClass) {
		    			$this.workflow = data[i];
		    			break;
		    		}
		    	}
		    	// trigger an event to specify the workflow is loaded
		    	$this.$element.trigger('loaded', $this.workflow);
		    	$this.$element.html("");
		    	$.tmpl($this.options.template, {workflow: $this.workflow, display_run_button: $this.options.displayRunButton,
		    		display_reset_button: $this.options.displayResetButton, parameters: $this.options.parameters, 
		    		getParameterDisplay:$this._getParameterDisplay, parameterTemplate: $this.options.parameterTemplate}).appendTo($this.$element);
		    	$('.date').datepicker().on('changeDate', function(ev){ $('.date').datepicker('hide');} ); 
		    	for (var i in $this.workflow.parameters) {
		    		// if it's a multiple parameter add its sub parameters as rules
		    		if ($this.workflow.parameters[i].type == "MultipleParameters") {
		    			for (var j in $this.workflow.parameters[i].sub_parameters) {
		    				rules[$this.workflow.parameters[i].sub_parameters[j].name] = $this._getParameterRule($this.workflow.parameters[i].sub_parameters[j], $this.options.serverURL);
		    			}
		    		} else {
		    			rules[$this.workflow.parameters[i].name] = $this._getParameterRule($this.workflow.parameters[i], $this.options.serverURL);
		    		}
		    		if ($this.workflow.parameters[i].group.indexOf("exclude-") != -1) {
		    			rules[$this.workflow.parameters[i].name]["exlude"] = "."+$this.workflow.parameters[i].group;
		    		}
		    	}
		    	$("#workflow_form").validate({ rules: rules });
		    	if ($this.options.displayRunButton) {
		    		$("#wfform_run_btn").click(function() { $this.run(); })
		    	}
		    	if ($this.options.displayResetButton) {
		    		$("#wfform_reset_btn").click(function() { $this.reset(); })
		    	}
		    }
    	});
	}
	
	WFForm.prototype._getParameterRule = function(param, server_url) {
		if (param.type == "int") {
			return {
	    		required: param.required,
	    		number: true
	    	}
		} else if (param.type == "date") {
			return {
	    		required: param.required,
	    		date: true
		    }
		} else if (param.type == "str") {
			return {
	    		required: param.required
		    }
		// if it is not a known type, check from the custom ones, using the validate_field address
		} else {
			return {
	    		required: param.required,
	    		remote: {
	    			url: server_url + '/validate_field?callback=?',
	    			type: "post",
	    			data: {
	    				type: param.type
	    			}
	    		}
	    	}
		}
	}
	
	WFForm.prototype._getParameterDisplay = function(param, template, input_size="input-xlarge", check_title="") {
    	return $.tmpl(template, {param:param, input_size:input_size, check_title:check_title}).html();
	}
	
	/* WFForm PLUGIN DEFINITION
	 * ========================== */	
	
	var old = $.fn.activewf

	$.fn.wfform = function (option) {
		return this.each(function () {
			var $this = $(this)
				, data = $this.data('wfform')
				, options = $.extend({}, $.fn.wfform.defaults, typeof option == 'object' && option)
				, action = typeof option == 'string' ? option : null
			// if already exist
			if (!data) { $this.data('wfform', (data = new WFForm(this, options))) }
			// otherwise change the workflow class
			else { data.options.workflowClass = options.workflowClass }
			if (action) { data[action]() }
			else { data.load(); }
		})
	}

	$.fn.wfform.defaults = {
		serverURL: "http://localhost:8080",
		parameterTemplate: ['<div> {{if param.choices}}',
			// if it's a multiple choice parameter, add a select
			'  <select id="${param.name}" name="${param.name}" class="${param.group} ${input_size}">',
			'  {{each(j, choice) param.choices}}',
			'    {{if choice == param.default}}',
			'      <option selected>${choice}</option>',
			'    {{else}}',
			'      <option>${choice}</option>',
			'    {{/if}}',
			'  {{/each}}',
			'  </select>',
			// if param is a date 
			'{{else param.type == "date"}}',
			'  <div class="input-append date" data-date="${param.default}" data-date-format="dd/mm/yyyy">',
			'    <input id="${param.name}" name="${param.name}" class="${param.group} ${input_size} span2" type="text" value="${param.default}">',
			'    <button class="btn" type="button"><i class="icon-calendar"></i></button>',
			'  </div>',
			// if param is a boolean 
			'{{else param.type == "bool"}}',
			'  <label class="checkbox">',
			'  {{if param.default == true}}',
			'    <input id="${param.name}" name="${param.name}" class="${param.group}" value="${param.default}" type="checkbox" checked> ${check_title}',
			'  {{else}}',
			'    <input id="${param.name}" name="${param.name}" class="${param.group}" value="${param.default}" type="checkbox"> ${check_title}',
			'  {{/if}}',
			'</label>',
			// else a simple text input
			'{{else}}',
			'  <input id="${param.name}" name="${param.name}" value="${param.default}" class="${param.group} ${input_size}" type="text">',
			'{{/if}}</div>'].join('\n'),
		template: ['<form id="workflow_form" class="form-horizontal">',
			'<fieldset>',
			'  {{each(gindex, group) workflow.groups}}',
			'    {{each(index, param) workflow.parameters_per_groups[group]}}',
			// it the parameter has not already been settled
			'    {{if Object.keys(parameters).indexOf(param.name) == -1 }}',
			'    <div class="control-group">',
			'      {{if group != "default" && group.indexOf("exclude-") != 0 && index == 0}}',
			'        <legend>${param.group}</legend>',
			'      {{/if}}',
			'      <label class="control-label">${param.display_name}</label>',
			'      <div class="controls">',
			// if it's a multiple type
			'      {{if param.type == "MultipleParameters"}}',
			'        <blockquote>',
			'        {{each(spindex, sub_param) param.sub_parameters}}',
			'          {{if sub_param.type == "bool"}}',
			'          {{html getParameterDisplay(sub_param, parameterTemplate, "input-medium", sub_param.display_name)}}',
			'          {{else}}',
			'          <div class="input-prepend">',
			'            <span class="add-on">${sub_param.display_name}</span>',
			'            {{html getParameterDisplay(sub_param, parameterTemplate, "input-medium")}}',
			'          </div>',
			'          {{/if}}',
			'          <span class="help-block">${sub_param.help}</span> <br />',
			'        {{/each}}',
			'        </blockquote>',
			'      {{else}}',
			'          {{html getParameterDisplay(param, parameterTemplate)}}',
			'          <span class="help-block">${param.help}</span>',
			'      {{/if}}',
			'      </div>',
			'    </div>',
			'    {{else}}',
			// if the param is already settled, hide it
			'    <input id="${param.name}" name="${param.name}" class="${param.group}" value="${parameters[param.name]}" type="hidden">',			
			'    {{/if}}',
			'    {{/each}}',
			'  {{/each}}',
			// for all workflow add the workflowClass
			'  <input name="workflow_class" value="${workflow.class}" type="hidden">',
			// add buttons if requested
			'{{if display_reset_button || display_run_button}}',
			'  <div class="control-group">',
			'    <label class="control-label"></label>',
			'    <div class="controls">',
			'      {{if display_reset_button}}',
			'      <button id="wfform_reset_btn" class="btn" type="button"><i class="icon-refresh"></i> Reset</button>',
			'      {{/if}}',
			'      {{if display_run_button}}',
			'      <button id="wfform_run_btn" class="btn btn-primary" type="button"><i class="icon-white icon-cog"></i> Run</button>',
			'      {{/if}}',
			'    </div>',
			'  </div>',
			'{{/if}}',
			'</fieldset>',
			'</form>'].join('\n'),
		workflowClass: null,
		displayRunButton: true,
		displayResetButton: true,
		workflow: null,
		parameters: {}
	}

	$.fn.wfform.Constructor = WFForm

}(window.jQuery);  