/***************************************************************
*  Copyright notice
*
*  (c) 2013 PF bioinformatique de Toulouse
*  All rights reserved
* 
*  It is distributed under the terms of the GNU General Public 
*  License as published by the Free Software Foundation; either 
*  version 2 of the License, or (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

!function ($) {

	"use strict"; // jshint ;_;


	/* WFStatus CLASS DEFINITION
	 * ========================= */

	var WFStatus = function (element, options) {
		this.$element = $(element);
		this.options = $.extend({}, $.fn.wfstatus.defaults, options);
		if (this.options.serverURL == "") { this.options.serverURL = $.fn.activewf.defaults.serverURL; }
	}
	
	WFStatus.prototype.reformatWorkflowJSON = function(data) {
		var workflow = {
			name: data.name, 
			status: data.status,
			elapsed_time: data.elapsed_time,
			end_time: data.end_time,
			id: data.id,
			start_time: data.start_time,
			components: new Array()
	    };
		for (var i in data.components) {
    		var failed = parseInt(((parseInt(data.components[i].aborted) + parseInt(data.components[i].failed)) * 100) / parseInt(data.components[i].total)),
    			completed = parseInt((parseInt(data.components[i].completed) * 100) / parseInt(data.components[i].total)),
    			running = parseInt((parseInt(data.components[i].running) * 100) / parseInt(data.components[i].total)),
    			waiting = parseInt((parseInt(data.components[i].waiting) * 100) / parseInt(data.components[i].total));
    		workflow.components.push({
    			name: data.components[i].name,
    			elapsed_time: data.components[i].elapsed_time,
    			failed: failed,
    			running: running,
    			completed: completed,
    			waiting: waiting
    		});
    	}
		return workflow;
	}
	
	WFStatus.prototype.reload = function() {
		if (this.options.forceUsingWorkflow) {
			var workflow = this.reformatWorkflowJSON(this.options.forceUsingWorkflow);
			this.$element.html("");
			$.tmpl(this.options.template, {workflow: workflow}).appendTo(this.$element);
			// force for first display, then reload an update
			this.options.forceUsingWorkflow = null;
		} else if (this.options.workflowID) {
			var $this = this,
				params = "",
				waiting_animation = ['<div class="row-fluid"><div class="span1 offset3"><div class="inline floatingBarsG">',
			                         '<div class="blockG" id="rotateG_01"></div>',
			                         '<div class="blockG" id="rotateG_02"></div>',
			                         '<div class="blockG" id="rotateG_03"></div>',
			                         '<div class="blockG" id="rotateG_04"></div>',
			                         '<div class="blockG" id="rotateG_05"></div>',
			                         '<div class="blockG" id="rotateG_06"></div>',
			                         '<div class="blockG" id="rotateG_07"></div>',
			                         '<div class="blockG" id="rotateG_08"></div>',
			                         '</div></div> <div class="span6">Please wait until modul is being loaded!</div></div>'].join('\n');
			$this.$element.html(waiting_animation);
			if (this.options.workflowID) { params = "workflow_id=" + this.options.workflowID + "&"; }
			$.ajax({
			    url: this.options.serverURL + '/get_workflows_status?'+params+'callback=?',
			    dataType: "json",
			    timeout: 20000,
			    error: function (xhr, ajaxOptions, thrownError) {
			    	var alert_message = ['<div class="alert alert-error">',
			        					'<strong>Error!</strong>',
			        					'Jflow failed to connect to the specified server <strong>' + $this.options.serverURL + '</strong>',
			        					'</div>'].join('\n');
			        $this.$element.html(alert_message);
			    },
			    success: function(data) {
			    	var workflow = $this.reformatWorkflowJSON(data);
			    	$this.$element.html("");
			    	$.tmpl($this.options.template, {workflow: workflow}).appendTo($this.$element);
			    }
	    	});
		} 
	}
	
	/* WFStatus PLUGIN DEFINITION
	 * ========================== */
	
	var old = $.fn.wfstatus

	$.fn.wfstatus = function (option) {
		return this.each(function () {
			var $this = $(this)
				, data = $this.data('wfstatus')
				, options = $.extend({}, $.fn.wfstatus.defaults, typeof option == 'object' && option)
				, action = typeof option == 'string' ? option : null
			// if already exist
			if (!data) $this.data('wfstatus', (data = new WFStatus(this, options)))
			// otherwise change the workflow
			else if (options.workflowID) { data.options.workflowID = options.workflowID; }
			if (options.forceUsingWorkflow) { data.options.forceUsingWorkflow = options.forceUsingWorkflow; }
			if (action) { data[action]() }
			else { data.reload(); }
		})
	}

	$.fn.wfstatus.defaults = {
		serverURL: "http://localhost:8080",
		template: ['<dl class="dl-horizontal">',
		    '<dt>Start</dt>',
		    '<dd>${workflow.start_time}</dd>',
		    '<dt>End</dt>',
		    '<dd>${workflow.end_time}</dd>',
	        '<dt>Elapsed time</dt>',
		    '<dd>${workflow.elapsed_time}</dd>',
		    '<br/>',
		    '{{each(index, component) workflow.components}}',
		    '<dt>${component.name}</dt>',
		    '<dd>',
		    '<div class="progress">',
		    '<div class="bar bar-success" style="width: ${component.completed}%;"></div>',
		    '<div class="bar bar-danger" style="width: ${component.failed}%;"></div>',
		    '<div class="bar" style="width: ${component.running}%;"></div>',
		    '<div class="bar bar-warning" style="width: ${component.waiting}%;"></div>',
		    '</div>',
		    '</dd>',
		    '{{/each}}',
		    '</dl>'].join('\n'),
		workflowID: null,
		forceUsingWorkflow: null
	}

	$.fn.wfstatus.Constructor = WFStatus

}(window.jQuery);  